# Test suite for LibIDN Ruby Bindings.
#
# Copyright (c) 2005 Erik Abele. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# Please see the file called LICENSE for further details.
#
# You may also obtain a copy of the License at
#
# * http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# This software is OSI Certified Open Source Software.
# OSI Certified is a certification mark of the Open Source Initiative.

require 'test/unit'

# Construct absolute project path.
#project_path = File.join(File.dirname(__FILE__), '..')

# Add project directories to global LOAD_PATH ($:).
#$:.unshift File.join(project_path, 'ext'), File.join(project_path, 'lib')

# Require all testcase files.
#Dir[File.join(project_path, 'test', 'tc_*.rb')].each { |tc| require tc }
